<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:f = "http://function"
    version="2.0">
    
    <xsl:output indent="yes"/>
    
    <xsl:template match="root">
        <BirdStrikes>
            <xsl:apply-templates/>
        </BirdStrikes>
    </xsl:template>
    
    <xsl:template match="row[elem[@name eq 'Record ID'] != '0']">
        <BirdStrike id="{elem[@name eq 'Record ID']}">
            <xsl:apply-templates/>
        </BirdStrike>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Altitude bin'][text()]" priority="2">
        <altitude unit="foot">
            <xsl:choose>
                <xsl:when test="starts-with(text(), '&lt;')">
                    <xsl:attribute name="type" select="'max'"/>
                    <xsl:copy-of select="replace(replace(text(), '&lt; ', ''), ' ft', '')"/>
                </xsl:when>
                <xsl:when test="starts-with(text(), '&gt;')">
                    <xsl:attribute name="type" select="'min'"/>
                    <xsl:copy-of select="replace(replace(text(), '&gt; ', ''), ' ft', '')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:copy-of select="replace(text(), ' ft', '')"/>
                </xsl:otherwise>
            </xsl:choose>
        </altitude>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Record ID']" priority="2"/>
    
    <xsl:template match="elem[@name eq 'FlightDate'][text()][text() ne '0']" priority="2">
        <xsl:variable name="dateThenTime" select="tokenize(text(), ' ')"/>
        <xsl:variable name="dateParts" select="tokenize($dateThenTime[1], '/')"/>
        <xsl:variable name="dt" select="concat(string-join(($dateParts[3], $dateParts[2], $dateParts[1]), '-'), 'T', $dateThenTime[2])"/>
        <xsl:if test="$dt castable as xs:dateTime">        
            <flight-date>
                <xsl:value-of select="xs:dateTime($dt)"/>
            </flight-date>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Speed (IAS) in knots'][text()]" priority="2">
        <speed unit="knot"><xsl:copy-of select="text()"/></speed>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Miles from airport'][text()]" priority="2">
        <distance-from-airport unit="mile"><xsl:copy-of select="text()"/></distance-from-airport>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Feet above ground'][text()]" priority="2">
        <elevation unit="foot"><xsl:copy-of select="text()"/></elevation>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Cost: Aircraft time out of service (hours)'][text()]" priority="2">
        <cost>
            <time-out-of-service>
                <xsl:value-of select="xs:dayTimeDuration(concat('PT', text(), 'H'))"/>
            </time-out-of-service>
        </cost>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Wildlife: Number struck'][text()]" priority="2">
        <wildlife>
            <number-struck>
                <xsl:choose>
                    <xsl:when test="contains(text(), ' to ')">
                        <xsl:variable name="minMax" select="tokenize(text(), ' to ')"/>
                        <xsl:attribute name="min" select="$minMax[1]"/>
                        <xsl:attribute name="max" select="$minMax[2]"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy-of select="text()"/>
                    </xsl:otherwise>
                </xsl:choose>
            </number-struck>
        </wildlife>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Remains of wildlife collected?'][text()]" priority="2">
        <wildlife>
            <remains>
                <collected><xsl:value-of select="text()"/></collected>
            </remains>
        </wildlife>
    </xsl:template>
    
    <xsl:template match="elem[@name eq 'Remains of wildlife sent to Smithsonian'][text()]" priority="2">
        <wildlife>
            <remains>
                <sent-to-smithsonian><xsl:value-of select="text()"/></sent-to-smithsonian>
            </remains>
        </wildlife>
    </xsl:template>
    
    <xsl:template match="elem[text()]">
        
        <xsl:choose>
            <xsl:when test="contains(@name,':')">
                
                
                    
                    <xsl:choose>
                        <xsl:when test="contains(@name, '(HHMM)')">
                            <xsl:variable name="time" select="f:hhmm-to-time(text())"/>
                            <xsl:if test="not(empty($time))">
                            <xsl:element name="{lower-case(substring-before(@name, ':'))}">
                                <xsl:element name="{replace(replace(replace(replace(lower-case(substring-after(@name, ': ')), ' ', '-'), '/', '-or-'), '\?', ''), '-\(hhmm\)', '')}">
                                    <xsl:value-of select="$time"/>
                                </xsl:element>
                            </xsl:element>
                            </xsl:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:element name="{lower-case(substring-before(@name, ':'))}">
                                <xsl:element name="{replace(replace(replace(replace(replace(lower-case(substring-after(@name, ': ')), ' ', '-'), '/', '-or-'), '\?', ''), '-\(inflation-adj\)', ''), '-\$', '')}">
                                    <xsl:if test="ends-with(@name, ' $')">
                                        <xsl:attribute name="currency" select="'USD'"/>
                                    </xsl:if>
                                    <xsl:value-of select="text()"/>
                                </xsl:element>
                            </xsl:element>
                        </xsl:otherwise>
                    </xsl:choose>
                
            </xsl:when>
            <xsl:otherwise>
                <xsl:element name="{replace(replace(replace(lower-case(@name), ' ', '-'), '\?', ''), '-\(ias\)', '')}">
                    <xsl:choose>
                        <xsl:when test="text() = 'Y'">true</xsl:when>
                        <xsl:when test="text() = 'N'">false</xsl:when>
                        <xsl:otherwise><xsl:copy-of select="text()"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose> 
    </xsl:template>
    
    
    <xsl:template match="node()|@*"/>
    
    <xsl:function name="f:hhmm-to-time" as="xs:time?">
        <xsl:param name="hhmm" as="xs:string"/>
        <xsl:choose>
            <xsl:when test="string-length($hhmm) le 2">
                <xsl:variable name="mins" select="if(string-length($hhmm) eq 1)then concat('0', $hhmm) else $hhmm"/>
                <xsl:value-of select="xs:time(concat('00:', $mins, ':00'))"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="mins" select="substring($hhmm, string-length($hhmm) - 1)"/>
                <xsl:variable name="hours" select="substring($hhmm, 1, string-length($hhmm) - 2)"/>
                <xsl:variable name="hours2" select="if(string-length($hours) eq 1)then concat('0', $hours) else $hours"/>
                <xsl:if test="xs:integer($hours) le 24 and xs:integer($mins) le 60">
                    <xsl:value-of select="xs:time(concat($hours2, ':', $mins, ':00'))"/>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    
</xsl:stylesheet>